import { getQuery } from 'mystique/hooks/getQuery';
import { FormFieldProps } from 'mystique/registry/FieldRegistry';
import { Connection } from 'mystique/types/common';
import { ChangeEvent, FC, useEffect, useState } from 'react';
import {
  FormControl,
  FormHelperText,
  InputLabel,
  MenuItem,
  Select,
} from '@material-ui/core';
import { Loading } from 'mystique/components/Loading';

export interface RetailerNode {
  id: number;
  tradingName: string;
}

interface RetailersResult {
  retailers: Connection<RetailerNode>;
}

const retailersQuery = `query {
  retailers {
    edges {
      node {
        id
        tradingName
      }
    }
  }
}
`;

interface RetailerId {
  id: number;
}

const RetailerSelector: FC<FormFieldProps<RetailerId | null>> = ({
  error,
  label,
  onBlur,
  onChange,
}) => {
  const [selectedOption, setSelectedOption] = useState<number | undefined>();
  const [retailers, setRetailers] = useState<RetailerNode[]>([]);
  const [loading, setLoading] = useState<boolean>(true);

  useEffect(() => {
    onChange(selectedOption ? { id: selectedOption } : null);
  }, [selectedOption]);

  useEffect(() => {
    getQuery<RetailersResult>(retailersQuery)
      .then((retailersData) => {
        const newRetailers: RetailerNode[] = [];
        retailersData.data?.retailers.edges.map((item) => {
          newRetailers.push(item.node);
        });
        setRetailers(newRetailers);
        setSelectedOption(newRetailers[0]?.id);
        setLoading(false);
      })
      .catch(() => {
        setLoading(false);
      });
  }, []);

  const handleChange = (event: ChangeEvent<{ value: unknown }>) => {
    setSelectedOption(event.target.value as number);
  };

  const handleOnBlur = () => {
    onBlur && onBlur();
  };

  return (
    <>
      {!loading ? (
        <FormControl fullWidth error={!!error}>
          <InputLabel id="retailers-select-label">{label}</InputLabel>
          <Select
            labelId="retailers-select-label"
            onChange={handleChange}
            value={`${selectedOption}`}
            onBlur={handleOnBlur}
          >
            {retailers.map((retailer: RetailerNode) => {
              return (
                <MenuItem key={retailer.id} value={`${retailer.id}`}>
                  {retailer.tradingName}
                </MenuItem>
              );
            })}
          </Select>
          {!!error && <FormHelperText> {error}</FormHelperText>}
        </FormControl>
      ) : (
        <Loading />
      )}
    </>
  );
};

export default RetailerSelector;
